
declare namespace magnoApi {

  function initAppList(): Promise<AppList>;
  function createApp(app: App): Promise<BaseResp>;
  function editApp(app: App, newApp: App): Promise<string | undefined>;
  function deleteApp(app: App): Promise<boolean>;
  function importApp(dir: string): Promise<string>;
  function canImport(dir: string): Promise<boolean>;
  function isAppExist(app: App): Promise<boolean>;
  function getAppConfig(app: App): Promise<AppConfig>;
  function setAppConfig(app: App, appConfig: AppConfig): Promise<string | undefined>;
  function getAssetConfig(app: App, asset: Asset): Promise<AssetConfigResp>;
  function setAssetConfig(asset: Asset, assetConfig: AssetConfig): Promise<boolean>;
  function queryAssetList(app: App, type: AssetType, refresh?: boolean): Promise<Asset[]>;
  function addToAssetFiles(app: App, asset: Asset): Promise<string | undefined>;
  function isAssetExist(app: App, asset: Asset): Promise<boolean>;
  function createAsset(app: App, asset: Asset): Promise<string | undefined>;
  function getWidgetNameList(app: App, refresh?: boolean): Promise<AssetListResp>;
  function downloadAsset(app: App, asset: Asset): Promise<BaseResp>;
  function buildAsset(app: App, asset: Asset): Promise<BaseResp>;
  function serveAsset(app: App, asset: Asset, open?: boolean): Promise<ServeAssetResp>;
  function stopAssetServer(asset: Asset): Promise<string | undefined>;  
  function publishAsset(app: App, asset: Asset, compile?: boolean, assetModel?: AssetModel): Promise<BaseResp>;
  function getLibraryList(app: App, refresh?: boolean): Promise<LibraryListResp>;
  function getBridgeConfig(libraryId: string): Promise<BridgeConfig>;
  function getAppAssetInfo(assetPath: string): Promise<AppAsset>;
  function getIndustry(): Promise<any>;
  function getCategeory(): Promise<any>;
  function getScenario(): Promise<any>;
  function getPluginDetailByIdentifier(libraryId: string): Promise<any>;
  function importAsset(app: App, assetPath: string): Promise<string | undefined>;  
  function deleteAsset(app: App, asset: Asset): Promise<string | undefined>;  
  function getRemoteEnv(): Promise<EnvInfo[]>;
  function setRemoteEnv(addEnv?: EnvInfo, deleteUrl?: string): Promise<EnvInfo[]>;
  function serveWidget(app: App, asset: Asset, open?: boolean): Promise<ServeAssetResp>;


  // 1: COMPONENT  2: WIDGET  3: WIDGETCOMPONENT
  type AssetType = 1 | 2 | 3;

  interface AppAsset {
    app?: App;
    asset?: Asset;
  }

  interface App {
    dir: string;
    name: string;
  }

  interface AppList {
    rootPath: string;
    apps?: App[];
  }


  interface AppConfig {
    url: string;
    byOauth: boolean;
    clientID?: string;
    clientSecret?: string;
    username?: string;
    password?: string;
    enableProxy: boolean;
    proxy?: Proxy;
  }

  interface Proxy {
    host: string;
    port: number;
    username?: string;
    password?: string;
  }

  interface Asset {
    type: AssetType;
    path?: string;
    dir?: string;
    name?: string;
    assetName?: string;
    assetDesc?: string;
    author?: string;
  }

  interface AssetModel {
    name?: any;
    releaseNote?: any;
    class?: string;
    industry?: string;
  }

  interface AssetConfigResp extends BaseResp {
    assetConfig?: AssetConfig;
  }

  interface AssetConfig {
    requires: Require[];
    properties?: any;
    bridgeProperties?: any;
    port?: number;
  }

  interface Require {
    name: string;
    version: string;
  }

  interface BaseResp {
    resCode: number;
    resMessage?: string;
    content?: string;
  }

  interface ServeAssetResp extends BaseResp {
    debugUrl?: string;
    previewUrl?: string;
  }

  interface AssetListResp extends BaseResp {
    remoteAssets?: RemoteAsset[];
  }

  interface RemoteAsset extends RemotePlugin {
    name: string;
    short?: string;
    value: string;
  }

  interface LibraryListResp extends BaseResp {
    remoteLibraries?: RemoteLibrary[];
  }

  interface RemoteLibrary extends RemotePlugin {
    libraryId?: string;
    name: string;
    version?: string;
  }

  interface RemotePlugin {
    pluginId?: string;
    pluginName: string;
    pluginVersion: string;
    pluginDescription?: string;
    pluginIcon?: string;
  }

  interface BridgeConfig {
    namespace?: string;
    staticData?: string;
    params?: BridgeConfigParams;
  }

  interface BridgeConfigParams {
    type: string;
    name: string;
    label?: string;
    options?: SelectOptions;
  }

  interface SelectOptions {
    label: string;
    value?: any;
    selected?: string;
  }

  interface EnvInfo {
    type?: string;
    name?: any;
    url: string;
  }
}

export = magnoApi;