id/*{manifest.widgetName}*/ = StudioWidgetWrapper.extend({
  /*
   * Triggered when initializing a widget and will have the code that invokes rendering of the widget
   * setParentContainer(JQueryParentContainerDOM) - binds event to this container
   * setItemContainer(JQueryItemContainerDOM) - binds studio item events for respective item containers
   * bindEvents() - binds the studio event to this widget
   */
  init: function() {
    var thisObj = this;
    thisObj._super.apply(thisObj, arguments);
    thisObj.render();
    if (typeof Studio != "undefined" && Studio) {
      [/*{ JSON.stringify(manifest.events).slice(1, -1) }*/].forEach(function (evt) {
        Studio.registerEvents(thisObj, evt.name, evt.displayName, evt.params);
      });
      [/*{ JSON.stringify(manifest.actions).slice(1, -1) }*/].forEach(function (act) {
        Studio.registerAction(
          thisObj,
          act.name,
          act.displayName,
          act.params,
          $.proxy(function(param) {
            thisObj.vm && thisObj.vm[act.function](param);
          }, thisObj),
          []
        );
      });
    }
  },

  /*
   * Triggered from init method and is used to render the widget
   */
  render: function() {
    var thisObj = this;
    var elem = thisObj.getContainer();

    /*
     * API to get base path of your uploaded widget API file
     */
    var widgetBasePath = thisObj.getWidgetBasePath();
    if (elem) {
      var loaded =
        "/*{manifest.entry}*/" in self
          ? $.Deferred().resolve()
          : $.getScript(widgetBasePath + "dist//*{manifest.entry}*/.umd.js");
      loaded.then(function() {
        var vueVm = self["/*{manifest.entry}*/"].init($(".widget",elem)[0], thisObj);
        if (vueVm && (vueVm.$children||[]).length) {
          thisObj.vm = vueVm.$children[0];
        }
      });
    }

    /*
     * API to bind global events to the item DOM :
     *  thisObj.sksBindItemEvent();
     *
     * API to bind item events to the item DOM :
     *  thisObj.sksBindItemEvent(JQueryItemContainerSelector, ItemIdx);
     * JQueryItemContainerSelector - A JQuery selector that returns an array of DOMs that represents the individual item inside the item container, to which the hotspot needs to be bound.
     * ItemIdx (Optional) - To bind the item events to a specific item.
     */
    thisObj.sksBindItemEvent();

    /*
     * API to refresh the previously bound events when a resize or orientation change occurs.
     *  thisObj.sksRefreshEvents(ItemIdx);
     * ItemIdx (Optional) - To refresh events for a specific item. Default value is 0.
     */
    $(window).resize(function() {
      thisObj.sksRefreshEvents();
    });
  }
});
