export default {
  methods: {
    //获取数据
    getData(apis, params, callBack) {
      //获取数据服务对象， 前提是用户需要props暴露options参数
      if (apis && apis.length > 0) {
        params = Array.isArray(params) ? params : []
        const promiseList = this.getPromiseList(apis, params)
        Promise.all(promiseList)
          .then(res => {
            if (typeof callBack === 'function') callBack(res)
          })
          .catch(error => {
            window.console.log(error)
          })
      }
    },
    //静态数据
    getstaticData(api) {
      return new Promise((resolve, reject) => {
        let getMapData = this.getFieldMap(api.source, api)
        resolve(getMapData)
      }).catch(error => {
        window.console.error('get static data failed')
      })
    },
    //通过api查询数据
    getAPIData(api, params = {}, headers = {}) {
      if (api && api.source && api.source.type && api.source.url) {
        return this.getNetWork(api, params, headers)
      }
    },
    /**
     * 获取数据集类型数据服务Promise - 支持数据集类型：静态数据，关系数据库，Roma API
     * @param {Object} api 卡片api参数
     */
    async getDataSetData(api, param = {}) {
      const { setId } = api.source

      if (!setId) {
        return
      }

      try {
        const newApi = JSON.parse(JSON.stringify(api))
        // 获取数据集信息
        const dataset = await this.getDMAXNetwork(
          `/magno/gallery/dataSet/${setId}`
        )
        if (!dataset) {
          return
        }
        return this.getDataSetValue(dataset, param ,newApi) 

      } catch (error) {
        window.console.log('get dataset data failed')
      }
    },
    async getDataSetValue(dataset,param,api){
      const response = await fetch(`/magno/gallery/dataSet/query/${dataset.id}`, {
        method: 'POST',
        credentials: 'same-origin',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify(param)
      });
      const data = await response.json();
     // 获取数据转换方法
     let transformer = null
     if (dataset.transformerId) {
       transformer = await this.getDMAXNetwork(`/magno/gallery/transFormer/${dataset.transformerId}`)
     }

      const outCode = (transformer && transformer.content) || '';
      if (!data || data.responseCode !== 0) {
        throw data
      }
      let result = this.transformCode('dataset', data.content, outCode);
      result = Array.isArray(result) ? result : [result]
      return  this.getFieldMap(result, api)
    },
    transformCode(paramName, data, code) {
      if (paramName && data && code) {
        const transformJsCode = new Function(paramName, code)
        return transformJsCode(data) || data
      } else {
        return data
      }
    },

    getNetWork(api, params = {}, headers = {}) {
      return new Promise((resolve, reject) => {
        let { url, type = 'GET', outCode } = api.source
        type = type.toLocaleUpperCase()

        if (url) {
          const requestParam = {
            method: type,
            cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
            credentials: 'same-origin', // include, same-origin, *omit
            headers: {
              'content-type': 'application/json',
              ...headers
            },
            mode: 'cors' // no-cors, cors, *same-origin
          }

          if (type === 'POST') {
            requestParam.body = params ? JSON.stringify(params) : {}
          } else if (type == 'GET') {
            url = this.formatUrlParams(url, params)
          }

          fetch(url, requestParam)
            .then(response => {
              return response.json()
            })
            .then(res => {
              // 出参转换
              let data = this.getDataInSelectRoot(res, outCode)

              data = Array.isArray(data) ? data : [data]
              const result = this.getFieldMap(data, api)
              resolve(result)
            })
            .catch(error => {
              reject(error)
            })
        }
      }).catch(error => {
        window.console.error('request data failed')
      })
    },
    /**
     *
     * @param {String} url
     * @param {String} type 请求方式
     * @param {Object} params 请求参数
     */
    async getDMAXNetwork(url, type = 'GET', params = {}) {
      type = type.toUpperCase()
      let requestParam = {
        method: type,
        cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
        credentials: 'same-origin', // include, same-origin, *omit
        headers: {
          'content-type': 'application/json'
        }
      }

      if (type === 'POST') {
        requestParam.body = JSON.stringify(params)
      } else if (type === 'GET') {
        url = this.formatUrlParams(url, params)
      }

      const resp = await fetch(url, requestParam)
      const data = await resp.json()
      if (!data || data.responseCode != 0) {
        throw data
      }
      return data.content
    },
    getFieldMap(source, api) {
      let data = source
      //处理数据映射
      if (api.fields && api.fields.length > 0) {
        data = data.map(i => {
          const newData = {}
          api.fields.forEach(field => {
            newData[field.field] = i[field.mapField]
          })
          return newData
        })
        return data
      }
    },
    //格式化入参,只有初始化时调用
    formatparmams(api) {
      let params = {}
      if (Array.isArray(api.params)) {
        api.params.map(item => {
          params[item.paramName] = item.paramDefaultValue
        })
      }
      return params
    },
    /**
     * 根据级联选择器选择的结果，取值
     * @param {Object} rs 返回结果
     * @param {Array} selectRoot 级联选择器选择结果
     */
    getDataInSelectRoot(rs, selectRoot) {
      let temp = null
      const selectRootData =
        selectRoot && typeof selectRoot === 'string'
          ? JSON.parse(selectRoot)
          : selectRoot
      const len = selectRootData.length
      if (Object.keys(rs).length > 0 && selectRoot && len > 0) {
        for (let i = 0; i < selectRootData.length; i++) {
          if (i === 0) {
            temp = rs[selectRootData[i]]
          } else {
            temp = temp[selectRootData[i]]
          }
        }
      } else {
        temp = rs
      }
      return temp
    },
    formatUrlParams(url, params) {
      const paramsArray = []
      for (const [key, value] of Object.entries(params)) {
        if (value || value === false) {
          paramsArray.push(`${key}=${value}`)
        }
      }
      if (paramsArray.length > 0) {
        if (url.search(/\?/) === -1) {
          url += `?${paramsArray.join('&')}`
        } else {
          url += `&${paramsArray.join('&')}`
        }
      }
      return url
    },
    // 未做定时器duration处理
    getPromiseList(apis, params = []) {
      const promiseList = []
      let promise
      apis.forEach((api, index) => {
        let { type } = api
        if (type === 'static') {
          promise = this.getstaticData(api)
          promiseList.push(promise)
        } else if (type === 'api') {
          if (api.source.url) {
            let param = this.formatparmams(api)
            promise = this.getAPIData(api, params[index] ? params[index] : param)
            promiseList.push(promise)
          }
        } else if (type === 'dataset') {
          if (api.source.setId) {
            promise = this.getDataSetData(api, params[index])
            promise && promiseList.push(promise)
          }
        }
      })
      return promiseList
    }
  }
}
