const artTemplate = require('art-template')
const json = require('../template/card')
const path = require('path')
const AdmZip = require('adm-zip')
const fs = require('fs')

const FIELDS = [{
  'field': 'date',
  'mapField': '',
  'id': 'f-one',
  'type': 'string',
  'desc': '时间'
},
{
  'field': 'whiteList',
  'mapField': '',
  'id': 'f-two',
  'type': 'integer',
  'desc': '白名单'
},
{
  'field': 'blackList',
  'mapField': '',
  'id': 'f-three',
  'type': 'integer',
  'desc': '黑名单'
}]

function convert(cardOptions) {
  let vueOptions = {}
  let initOptions = {}
  let noEditable = []
  if (cardOptions.definition && Array.isArray(cardOptions.definition)) {
    cardOptions.definition.filter(item => item.value.category === 0).map(item => {
      vueOptions[item.key] = item.value.value
    })

    cardOptions.definition.filter(item => item.value.category === 1).map(item => {
      initOptions[item.key] = item.value.value
    })

    noEditable = cardOptions.definition.filter(item => item.value.editable === false).map(item => item.key)
  }

  let options = {}
  options.apis = cardOptions.dataService && cardOptions.dataService.componentList

  const dimensionList = cardOptions.dataService && cardOptions.dataService.componentList && cardOptions.dataService.componentList[0].dimensionList
  
  options.apis[0].fields = FIELDS.map(item => {
    return {
      ...item,
      mapField: dimensionList[item.field]
    }
  })

  json.name = cardOptions.name
  json.title = cardOptions.title
  json.icon = cardOptions.icon
  noEditable.map(item => {
    delete json.definition.props.properties[item]
  })

  json.initProps = Object.assign(json.initProps, initOptions)
  json.initProps.options = Object.assign(json.initProps.options, options)

  let zip = new AdmZip()
  let vue = artTemplate(path.join(__dirname, '../template/Card.vue'), vueOptions || {})
  zip.addFile('Card.vue', Buffer.from(vue, 'utf8'))
  // customPanel配置的卡片，参数不会通过模板渲染（category === 1），直接打包vue，否者模板渲染会报错
  // zip.addLocalFile(path.join(__dirname, '../template/Card.vue'))
  zip.addFile('card.json', Buffer.from(JSON.stringify(json), 'utf8'))
  zip.addLocalFile(path.join(__dirname, '../template/index.js'))
  if (fs.existsSync(path.join(__dirname, '../template/customPanel'))) {
    zip.addLocalFolder(path.join(__dirname, '../template/customPanel'), 'customPanel')
  }
  zip.addLocalFolder(path.join(__dirname, '../template/static'), 'static')
  zip.writeZip(path.join(__dirname, '../template/card.zip'))

  return { code: 1, zipPath: path.join(__dirname, '../template/card.zip') }
}

module.exports = { convert }
