const express = require('express')
const morgan = require('morgan')
const bodyParser = require('body-parser')
const cookieParser = require('cookie-parser')
const https = require('https')
const fs = require('fs')

const app = express()

// set access log format
if (process.env.NODE_ENV != 'test') {
  app.use(morgan(':remote-addr - :remote-user [:date[clf]] :method :url HTTP/:http-version :status :res[content-length] :response-time'))
}

// parse body params and attache them to req.body
app.use(bodyParser.json({ limit: '50mb' }))
app.use(bodyParser.urlencoded({ limit: '50mb', extended: true }))

app.use(cookieParser())

const allowCrossDomain = function (req, res, next) {
  res.header('Access-Control-Allow-Origin', '*')// 自定义中间件，设置跨域需要的响应头。
  res.header('Access-Control-Allow-Headers', '*')
  next()
}

app.use(allowCrossDomain)// 运用跨域的中间件

const routes = require('./index.route')
app.use('/', routes)

const host = process.env.HOST || '0.0.0.0'
const port = process.env.PORT || 3000

// app.listen(port, host, () => console.log(`server started on ${host}:${port}`))

// 同步读取密钥和签名证书
const options = {
  key: fs.readFileSync('./keys/server.key'),
  cert: fs.readFileSync('./keys/server.crt')
}

// 创建服务
let httpsServer = https.createServer(options, app)

// //https监听3000端口
httpsServer.listen(port, host, () => {
  console.log(`server started on ${host}:${port}`)
})
